package client;

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;

import java.io.*;
import java.util.Date;

import common.ChatClient;
import common.ChatServer;
import common.Message;


public class ChatClientImpl extends UnicastRemoteObject
    implements ChatClient, Runnable {

    private ChatServer mycs;

    public ChatClientImpl( ChatServer cs ) throws java.rmi.RemoteException {
	mycs = cs;
	mycs.register( this );
    }


    public synchronized void receive( Message m ) throws java.rmi.RemoteException {
	System.out.println( m );
    }


    public void run() {
	BufferedReader ir
	    = new BufferedReader(new InputStreamReader(System.in));
	String msgtext;
	Message msg;

	Runtime.getRuntime().addShutdownHook( new Thread() {
		public void run() {
		    try {
			mycs.deregister( ChatClientImpl.this );
		    } catch( java.rmi.RemoteException re ) {
			System.err.println("Problem with deregistering from server!");
			re.printStackTrace();
		    }
		}
	    } );

	while( true ) {
	    try {
		System.out.println("Enter a message sometime...");
		msgtext = ir.readLine();

		msg = new Message( this.toString(), new Date(), msgtext );
		System.out.println("Broadcasting message: " + msgtext );

		mycs.broadcast( msg );
	    } catch( Exception e ) {
		System.err.println("Client had a problem...");
		e.printStackTrace();
	    }
	}
    }


    public static void main( String[] args ) {
	String url = "rmi://gem/ChatServer";

	try {
	    ChatServer cs = (ChatServer) Naming.lookup( url );
	    new Thread(new ChatClientImpl( cs )).start();
	} catch( Exception e ) {
	    System.err.println("Client had a startup problem...");
	    e.printStackTrace();
	}

    }

}
