package server;

import java.util.*;
import java.io.*;
import java.net.*;

import common.Message;


public class SocketChatServer {

    class ServiceThread extends Thread {
	private Socket s;

	public ServiceThread( Socket p ) {
	    s = p;
	}

	public void run( ) {
	    try {
		ObjectInputStream ois = new ObjectInputStream( s.getInputStream() );
		String cmd = (String) ois.readObject();
	    
		if ( cmd.equals("register") ) {
		    InetAddress host = (InetAddress) ois.readObject();
		    int port = ois.readInt();

		    InetSocketAddress sa = new InetSocketAddress(host, port);
		    SocketChatServer.this.register( sa );
		} else if ( cmd.equals("deregister") ) {
		    InetAddress host = (InetAddress) ois.readObject();
		    int port = ois.readInt();

		    InetSocketAddress sa = new InetSocketAddress(host, port);
		    SocketChatServer.this.deregister( sa );
		} else if ( cmd.equals("bcast") ) {
		    Message msg = (Message) ois.readObject();
		    
		    SocketChatServer.this.broadcast( msg );
		} else {
		    System.err.println("Unknown command!");
		}
		ois.close();
		s.close();
	    } catch( Exception ioe ) {
		System.err.println("Error while decoding command!");
		ioe.printStackTrace();
	    }
	}
    }


    // the addresses of all the clients
    private List<SocketAddress> myclients;


    public SocketChatServer() {
	myclients = new LinkedList<SocketAddress>();
    }


    public synchronized void register( SocketAddress sa ) {

	System.out.println("Registering a new client...");
	myclients.add( sa );
    }

    public synchronized void deregister( SocketAddress sa ) {

	System.out.println("Deregistering a client...");
	myclients.remove( sa );
    }


    public synchronized void broadcast( Message m ) {
	ListIterator<SocketAddress> it = myclients.listIterator(0);
	SocketAddress csa;
	int i = 0;

	System.out.println("Broadcast called...");

	while( it.hasNext() ) {
	    csa = it.next();

	    System.out.println("Sending message to client " + ++i);
	    sendMessage( csa, m );
	}
    }

    private void sendMessage( SocketAddress csa, Message m ) {
	try {
	    Socket s = new Socket();
	    s.connect( csa, 10000 );
	    ObjectOutputStream oos = new ObjectOutputStream( s.getOutputStream( ) );
	    oos.writeObject( "msg" );
	    oos.writeObject( m );
	    oos.close();
	    s.close();
	} catch( IOException ioe ) {
	    System.err.println("Error while trying to send a message!");
	    ioe.printStackTrace();
	}
    }


    public static void main( String[] args ) {
	try {
	    System.out.println("Starting server...");
	    SocketChatServer srv = new SocketChatServer();
	    ServerSocket ss = new ServerSocket( 6666 );

	    while ( true ) {
		Socket s = ss.accept( );
		srv.new ServiceThread( s ).start();
	    }
	} catch( Exception ex ) {
	    System.err.println("Something in the server failed!");
	    ex.printStackTrace();
	}
    }

}
